#!/usr/bin/perl
#
# Generate the video character generator portion in Altera MIF format
#

eval { use bytes; };

@arrays = ('chgen0', 'chgen1', 'chgen2', 'chgen3');

@chars = ( "                                ",
           " !\"#%&\'()*+,-./0123456789:;<=>?",
	   "ABCDEFGHIJKLMNOPQRSTUVWXYZ_",
	   "abcdefghijklmnopqrstuvwxyz\x7f" );

while ( defined($line = <STDIN>) ) {
    chomp $line;
    if ( $line =~ /^FONTBOUNDINGBOX\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ ) {
	$xsize = $1 + 0;
	$ysize = $2 + 0;
	$xoff  = $3 + 0;
	$yoff  = $4 + 0;
	@bitmasks = ("\0" x $ysize) x 256;
    } elsif ( $line =~ /^ENCODING\s+(\S+)/ ) {
	$thischar = $1 + 0;
    } elsif ( $line =~ /^BBX\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/ ) {
	$cxsize = $1 + 0;
	$cysize = $2 + 0;
	$xstart = $3 - $xoff;
	$ystart = $ysize - $4 - $cysize + $yoff;
    } elsif ( $line =~/^BITMAP\b/ ) {
	for ( $i = 0 ; $i < $cysize ; $i++ ) {
	    $line = <STDIN>;
	    substr($bitmasks[$thischar], $i+$ystart, 1) =
		chr((hex $line) >> $xstart);
	}
    }
}

# Print out the initialization vectors for the screen fonts
# Each RAM array holds 32 characters and is dumped into 16 chunks

print "DEPTH = 2048;\n";
print "WIDTH = 8;\n";
print "ADDRESS_RADIX = HEX;\n";	# BIN, DEC, HEX, OCT, or UNS
print "DATA_RADIX    = HEX;\n";
print "CONTENT\n";
print "BEGIN\n";

$a = 0;
foreach $rom ( @arrays ) {
    for ( $i = 0 ; $i < 16 ; $i++ ) {
	for ( $j = 0 ; $j < 2 ; $j++ ) {
	    $char = $bitmasks[ord(substr($chars[0],$i*2+$j,1))];
	    for ( $k = 0 ; $k < 16 ; $k++ ) {
		printf "%04X : %02X;\n", $a++, ord(substr($char,$k,1));
	    }
	}
    }
    shift @chars;
}
print "END;\n";
